-- Kleines "Framework" zum anwenden der BW-Transformation auf Dateien

module Transform where

import BWT
import Time 
	
-- Transformieren des Inhalts einer Datei mit "Transformations-Algo-Auswahl"
-- -> fi: Name der Inputdatei
-- -> fo: Name der Outputdatei
-- -> f:  Transformation (transform | transform2)
ftransformation :: String -> String -> (String -> (String,Int))-> IO ()
ftransformation fi fo f = do
                          stime <- getClockTime
                          s <- readFile fi
                          writeFile fo (show (f s)) 
                          etime <- getClockTime
                          td <- (return .timeDiff2String . diffClockTimes etime) stime
                          putStr ("Transformation successful within " ++ td ++ "\n")
		
-- Rck-Transformieren des Inhalts einer Datei mit "Rck-Transformations-Algo-Auswahl"
-- -> fi: Name der Inputdatei
-- -> fo: Name der Outputdatei
-- -> f:  Transformation (untransform | untransform2 | untransform3 | untransform4)
funtransformation :: String -> String -> ((String,Int) -> String)-> IO ()	
funtransformation fi fo f = do
                     stime <- getClockTime
                     s <- readFile fi
                     ut <- return (f ((read s)::(String,Int)))
                     writeFile fo ut
                     etime <- getClockTime
                     td <- (return .timeDiff2String . diffClockTimes etime) stime
                     putStr ("Untransformation successful within " ++ td ++ "\n")

-- Wandelt eine TimeDiff in einen String um, 
-- es werden allerdings nur Sekunden und PikoSekunden in den Ausgabestring geschrieben
timeDiff2String :: TimeDiff -> String					 
timeDiff2String = merge.fork (show . tdSec,show . tdPicosec)
                 where
                 fork (f,g) x = (f x,g x)
                 merge (a,b) = a ++ " Sec " ++ b ++ " PSec"
				

-- Transformieren des Inhalts einer Datei
-- -> fi: Name der Inputdatei
-- -> fo: Name der Outputdatei							
ftransform :: String -> String -> IO ()
ftransform fi fo = ftransformation fi fo (transform2 '~')

-- Rck-Transformieren des Inhalts einer Datei
-- -> fi: Name der Inputdatei
-- -> fo: Name der Outputdatei					 
funtransform :: String -> String -> IO ()	
funtransform fi fo = funtransformation fi fo untransform4

main :: IO()
main = do
       ftransform "src_wedel.txt" "t_wedel.txt"
       funtransform "t_wedel.txt" "ut_wedel.txt"
       ftransform "src_lorem.txt" "t_lorem.txt"
       funtransform "t_lorem.txt" "ut_lorem.txt"
